import java.util.Iterator;

public class Composition<T, U extends Connection<T>> extends Ensemble<T> implements Connection<Composition<?,Connection<?>>>, Supply { //Composition<? extends Compositable, ? extends Connection>
    private String name;
    private Ensemble<Connection<? extends T>> connections;
    private Composition<?, Connection<?>> source, sink;
    private Ensemble<Connection<Composition<?,Connection<?>>>> compConnections;

    //pre: name is displayed in toString
	public Composition(String name) {
        this.name = name;
        this.compConnections = new Ensemble<>(true);
        this.connections = new Ensemble<>(true);
	}

	//post: adds x to connections, adds x.sink() to supplies, adds x.source() to supplies
    public void connect(Connection<? extends T> x) {
        this.connections.add(x);
	    this.add(x.sink());
        this.add(x.source());
	}

	//post: adds x to connections, sets x.sink() to this connections sink, sets x.source() to this connections source
	public void compConnect(Connection<Composition<?, Connection<?>>> x) {
        this.source = x.source();
        this.sink = x.sink();
        this.compConnections.add(x);
    }

    public Iterator<Connection<? extends T>> connectionIter() { return this.connections.iterator(); }

    public Iterator<Connection<Composition<?,Connection<?>>>> compConnectionIter() { return this.compConnections.iterator(); }

    public Composition<?, Connection<?>> source() {
        return this.source;
    }

    public Composition<?, Connection<?>> sink() {
        return this.sink;
    }

    public float property() {
        return 0;
    }

    //post: returns a string representation of this. If normal connections are available those will be printed;
    //else this Composition is a Connection and those will be printed
    //same goes for source and sink. Only if they are available they will be printed
    public String toString() {
	    String connections = this.connections.iterator().hasNext() ? this.connections.toString() : this.compConnections.toString();
	    String sourceString = (this.source == null) ? "" : "\n\t\tSource: " + this.source.toString();
	    String sinkString = (this.sink == null) ? "" : "\n\t\tSink: " +this.sink.toString();
	    return  "Composition("+this.name+
                "): Connections" + connections
                + "\n\t\tSupplies: " + super.toString()
                + sourceString + sinkString;
	}
}
