public class Ensemble<T> implements java.lang.Iterable<T> {

    // default constructor - no duplicates in this Ensemble allowed
    public Ensemble() {
        this.dupes = false;
    }
    // constructor to set duplicate option manually
    // pre: dupes true => duplicates allowed
    public Ensemble(boolean dupes) {
        this.dupes = dupes;
    }
    private boolean dupes;

    // represents one Node-element of this Ensemble
    private class Node {
        private T elem;
        private Node next = null;
        private Node(T elem) {
            this.elem = elem;
        }
    }

    private Node head = null, tail = null;

    private class EnsembleIter implements java.util.Iterator<T> {
        private Node p = head;
        private Node pre = null, prepre = null;

        // post: returns the element from the next Node of this iterator
        public T next() {
            if (p == null) return null;
            T elem = p.elem;
            prepre = pre;
            pre = p;
            p = p.next;
            return elem;
        }

        // post: returns true if there is a next element in this iterator that's != null
        public boolean hasNext() { return p != null; }

        // post: Removes the current (last element from next()) element from this iterator
        public void remove() {
            if(prepre == null) {
                head = p;
                pre = null;
            }
            else if(p == null) {
                prepre.next = null;
                tail = prepre;
            }
            else {
                prepre.next = p;
                pre = prepre;
            }
        }
    }

    // pre: toadd has to be not null
    // post: Adds toadd to ensemble
    //       allows duplicates if (boolen) dupes = true
    //       otherwise duplicates are not added to this Ensemble
    public <U extends T> void add(U toadd) {
        //TODO Make sure it doesn't add dupes
        if(toadd == null) {
            return;
        }
        if(head == null) {
            head = tail = new Node(toadd);
        } else {
            if (!dupes) {
                Node c = head;
                if (head.elem == toadd) {
                    return;
                }
                while ((c = c.next) != null) {
                    if (c.elem == toadd) {
                        return;
                    }
                }
            }
            tail = tail.next = new Node(toadd);
        }
    }

    // post: returns a new iterator from this Ensemble
    public java.util.Iterator<T> iterator() { return new EnsembleIter(); }

    // "(head -> elem -> elem -> tail)"
    public String toString() {
  		String res = "[()";
		for(T s : this) {
			res += " -> (" + s.toString() + ")";
		}
		res += "]";
		return res;
	}
}
